pub enum LossFunction {
    Custom(CustomLossFunction),
    Stock(StockLossFunction),
}
Loss function for ResidualBlock and CurveFitProblem1D, it is a transformation of the squared residuals which is generally used to make the solver less sensitive to outliers. This enum has two flavours: user specified function and Ceres stock function.

Variants



Custom(CustomLossFunction)

User-specified loss function.



Stock(StockLossFunction)

One of the loss functions specified by Ceres, see Ceres Solver docs for details.

Implementations

Create a LossFunction to handle a custom loss function.

Arguments

Calls the underlying loss function.