pub enum ProblemStatus<'cost> {
    Uninitialized,
    Ready {
        blocks: Vec<ProblemBlock<'cost>>,
    },
    Solved,
}
NllsProblem’s state

Variants



Uninitialized

The problem is blank, at least one residual block must be added via NllsProblem::add_residual_block.



Ready

Fields

blocks: Vec<ProblemBlock<'cost>>

The problem is ready for NllsProblem::solve call, but you may add more residual blocks via NllsProblem::add_residual_block.



Solved

The problem is solved, drop this one and create a new.