Struct ceres_solver::nlls_problem::NllsProblem
source · pub struct NllsProblem<'cost> { /* private fields */ }Expand description
Non-Linear Least Squares problem.
See module-level documentation building the instance of this type.
Implementations
sourceimpl<'cost> NllsProblem<'cost>
impl<'cost> NllsProblem<'cost>
sourcepub fn residual_block_builder(self) -> ResidualBlockBuilder<'cost>
pub fn residual_block_builder(self) -> ResidualBlockBuilder<'cost>
Capture this problem into a builder for a new residual block.
sourcepub fn set_parameter_block_constant(
&mut self,
block_index: usize
) -> Result<(), ParameterBlockStorageError>
pub fn set_parameter_block_constant( &mut self, block_index: usize ) -> Result<(), ParameterBlockStorageError>
Set parameter block to be constant during the optimization. Parameter block must be already added to the problem, otherwise ParameterBlockStorageError returned.
sourcepub fn set_parameter_block_variable(
&mut self,
block_index: usize
) -> Result<(), ParameterBlockStorageError>
pub fn set_parameter_block_variable( &mut self, block_index: usize ) -> Result<(), ParameterBlockStorageError>
Set parameter block to be variable during the optimization. Parameter block must be already added to the problem, otherwise ParameterBlockStorageError returned.
sourcepub fn is_parameter_block_constant(
&self,
block_index: usize
) -> Result<bool, ParameterBlockStorageError>
pub fn is_parameter_block_constant( &self, block_index: usize ) -> Result<bool, ParameterBlockStorageError>
Check if parameter block is constant. Parameter block must be already added to the problem, otherwise ParameterBlockStorageError returned.
sourcepub fn solve(
self,
options: &SolverOptions
) -> Result<NllsProblemSolution, NllsProblemError>
pub fn solve( self, options: &SolverOptions ) -> Result<NllsProblemSolution, NllsProblemError>
Solve the problem.