pub struct Parameters { /* private fields */ }
ResidualBlock’s internal representation of a problem parameters.

Implementations

Creates a new Parameters from a vector of parameters, keeping in mind that each parameter is a f64 vector itself.

Number of parameters.

Parameter sizes.

Convert to vector of parameters, each parameter is vector of floats.