pub struct NllsProblem<'cost> { /* private fields */ }
Non-Linear Least Squares problem.

You use it in three steps:

NllsProblem::solve call invalidates the NllsProblem instance, so calling this method twice would return Err. Err also is returned if no residual block have been added to the problem.

Implementations

Adds a residual block to the problem.

Returns Err if NllsProblem has ProblemStatus::Solved state.

Solves the problem and returns a vector of solutions, one per residual block. Returns Err if problem doesn’t have ProblemStatus::Ready state.

Returns the status of the problem.

Returns the number of residual blocks for an unsolved problem, zero otherwise.